; These are equates, they work exactly like #define in C
; We use them here for the locations of the various data in VU memory
ProjMat			.equ  	0
Scale			.equ	4
NumVertsMem		.equ	4
GIFTag			.equ    5
StartSTQ		.equ    6
StartRGBA		.equ	7
StartVert		.equ	8

; This is a file full of macros that comes with VCL
.include "vcl_sml.i"

; Tell VCL that we want to be able to use all vf, and vi registers
.init_vf_all
.init_vi_all

.syntax new
.vu

; This is the entry point, and is where execution will start on VU1
--enter
--endenter

	fcset		0x000000					; VCL won't let us use CLIP without first zeroing
											; the clip flags

	iaddiu		iVert, vi00, 0				; Start vertex counter
	iaddiu		iVertPtr, vi00, 0			; Point to the first vert
	
	ilw.w		iNumVerts, NumVertsMem(vi00); We now load the number of verts from memory
											; instead of using a .equ

	lq			fScales, Scale(vi00)		; A scale vector that we will add
											; to and scale the vertices by after
											; projecting them
	
	; Load the transformation matrix
	lq			fTransform[0], ProjMat+0(vi00)
	lq			fTransform[1], ProjMat+1(vi00)
	lq			fTransform[2], ProjMat+2(vi00)
	lq			fTransform[3], ProjMat+3(vi00)
	
loop:										; For each vertex
	lq			Vert, StartVert(iVertPtr)	; Load the vector (currently in object space
											; and floating point format)
	
	; Transform the vertex by the projection matrix we passed in (this is a macro from
	; vcl_sml.i)
	MatrixMultiplyVertex Vert, fTransform, Vert
	
	; *** NEW ***
	; CLIPPING
	clipw.xyz	Vert, Vert					; This instruction checks if the vertex is outside
											; the viewing frustum. If it is, then the appropriate
											; clipping flags are set
	fcand		vi01, 0x3FFFF				; Bitwise AND the clipping flags with 0x3FFFF, this makes
											; sure that we get the clipping judgement for the last three
											; verts (i.e. that make up the triangle we are about to draw)
											; A true result of fcand will put 1 into vi01.
											; A false result of fcand will put 0 into vi01 
	iaddiu		iADC, vi01, 0x7FFF			; Add 0x7FFF. If any of the clipping flags were set this will
											; cause the triangle not to be drawn (a value of 0x8000
											; that is stored in the w component of XYZ2 will set the ADC
											; bit, which tells the GS not to perform a drawing kick on this
											; triangle). 
	isw.w		iADC, StartVert(iVertPtr)	; Finally store the ADC bit back to memory
	; END CLIPPING
    
    div         q,    vf00[w], Vert[w]		; Work out 1.0f / w
    mul.xyz     Vert, Vert, q				; Project the vertex by dividing by W
    
    lq			STQ,  StartSTQ(iVertPtr)	; Load the texture coordinates
    mul			STQ,  STQ, q				; Divide by W
    sq			STQ,  StartSTQ(iVertPtr)	; Store the texture coordinates back
    
    mula.xyz	acc, fScales, vf00[w]		; Move fScales into the accumulator (acc = fScales * 1.0)
 	madd.xyz	Vert, Vert, fScales			; Multiply and add the scales (Vert = Vert * fScales + fScales)
	ftoi4.xyz	Vert, Vert					; Convert the vertex to 12:4 fixed point for the GS
	
	sq.xyz		Vert, StartVert(iVertPtr)	; Write the vertex back to VU memory
	
	lq			RGBA, StartRGBA(iVertPtr)	; Load the RGBA colour
	ftoi0		RGBA, RGBA					; Convert to integer
	sq			RGBA, StartRGBA(iVertPtr)	; Write back to VU Memory

	iaddiu		iVert, iVert, 1				; Increment the vertex counter
	iaddiu		iVertPtr, iVertPtr, 3		; Increment the vertex pointer
	ibne		iVert, iNumVerts, loop		; Branch back to "loop" label if
											; iVert and iNumVerts are not equal
											
	iaddiu		iGIFTag, vi00, GIFTag		; Load the position of the giftag
	xgkick		iGIFTag						; and tell the PS2 to send that to the GS
	
--exit
--endexit
